@@ -1,6 +1,7 @@
 from __future__ import annotations
 import io
 import os
+import pathlib
 import sys
 import subprocess
 import ctypes
@@ -13,10 +14,12 @@ from .envars import (
     CF_TYPY_MODE,
     CF_TYPY_MODE_JULIA,
     CF_TYPY_MODE_PYTHON,
+    CF_TYPY_PID,
     CF_TYPY_PY_APIPTR,
     TyPython_directory,
     InitTools_path,
     SessionCtx,
+    jl_opts_parse,
 )
 
 # XXX: adding an environment variable for fast debugging:
@@ -94,38 +97,63 @@ class JuliaError(Exception):
     pass
 
 
-def init_jl():
+def init_jl(experimental_fast_init=False):
+    # if experimental_fast_init is True, assume TyPython is included in sysimage
     global _eval_jl
     if os.getenv(CF_TYPY_MODE) == CF_TYPY_MODE_JULIA:
         return
     elif os.getenv(CF_TYPY_MODE) == CF_TYPY_MODE_PYTHON:
         assert os.getenv(CF_TYPY_PY_APIPTR, str(ctypes.pythonapi._handle))
+        try:
+            assert os.getenv(CF_TYPY_PID) == str(os.getpid())
+        except AssertionError:
+            del os.environ[CF_TYPY_MODE]
+            del os.environ[CF_TYPY_PY_APIPTR]
+            del os.environ[CF_TYPY_PID]
+            init_jl()
         return
     elif not os.getenv(CF_TYPY_MODE):
         os.environ[CF_TYPY_MODE] = CF_TYPY_MODE_PYTHON
         os.environ[CF_TYPY_PY_APIPTR] = str(ctypes.pythonapi._handle)
+        if not os.getenv(CF_TYPY_PID):
+            os.environ[CF_TYPY_PID] = str(os.getpid())
     else:
         raise Exception("Unknown mode: " + (os.getenv(CF_TYPY_MODE) or "<unset>"))
 
     setup_julia_exe_()
     jl_opts = shlex.split(os.getenv(CF_TYPY_JL_OPTS, ""))
     jl_opts_proj = get_project_args()
-    cmd = [
-        SessionCtx.JULIA_EXE,
-        jl_opts_proj,
-        *jl_opts,
-        "--startup-file=no",
-        "-O0",
-        "--compile=min",
-        "-e",
-        julia_info_query,
-    ]
+    opts, unkown_opts = jl_opts_parse.parse_known_args([jl_opts_proj, *jl_opts]) # parse arg --sysimage or -J
+    if experimental_fast_init:
+        cmd = [
+            SessionCtx.JULIA_EXE,
+            *unkown_opts,
+            "--startup-file=no",
+            "-O0",
+            "--compile=min",
+            "-e",
+            julia_info_query,
+        ]
+    else:
+        cmd = [
+            SessionCtx.JULIA_EXE,
+            jl_opts_proj,
+            *jl_opts,
+            "--startup-file=no",
+            "-O0",
+            "--compile=min",
+            "-e",
+            julia_info_query,
+        ]
     bindir, libpath, sysimage, default_project_dir = subprocess.run(
         cmd, check=True, capture_output=True, encoding="utf8"
     ).stdout.splitlines()
-    SessionCtx.JULIA_START_OPTIONS = [jl_opts_proj, *jl_opts]
+    SessionCtx.JULIA_START_OPTIONS = unkown_opts
     SessionCtx.DEFAULT_PROJECT_DIR = default_project_dir
 
+    if experimental_fast_init and opts.sysimage:
+        sysimage_abs_paths = pathlib.Path(opts.sysimage).absolute().as_posix()
+        sysimage = sysimage_abs_paths
     old_cwd = os.getcwd()
     try:
         os.chdir(os.path.dirname(os.path.abspath(libpath)))
